# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.widgets.BaseBook.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTK Book Meta-Class."""

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.views.gtk3 import GObject, Gtk


class RAMSTKBaseBook(Gtk.Notebook):
    """The RAMSTK Book meta-class.

    Attributes of the Base Book are:

    :cvar dict dictab_position: dictionary containing the available
        Gtk.Notebook tab positions and associated noun name.
    :ivar RAMSTK_USER_CONFIGURATION: the RAMSTKUserConfiguration class
        instance.
    """

    RAMSTK_SITE_CONFIGURATION = None

    dic_tab_position = {
        "left": Gtk.PositionType.LEFT,
        "right": Gtk.PositionType.RIGHT,
        "top": Gtk.PositionType.TOP,
        "bottom": Gtk.PositionType.BOTTOM,
    }

    def __init__(self, configuration: RAMSTKUserConfiguration) -> None:
        """Initialize an instance of the Module Book class.

        :param configuration: the RAMSTKUserConfiguration class instance.
        """
        GObject.GObject.__init__(self)  # pylint: disable=non-parent-init-called

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.
        self.dic_handler_id = {"": 0}

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.RAMSTK_USER_CONFIGURATION = configuration

        # Subscribe to PyPubSub messages.

    def _set_properties(self, book: str) -> None:
        """Set properties of the RAMSTK Books and widgets.

        :param book: which book to set properties for.
        :return: None
        :rtype: None
        """
        try:
            _tab_position = self.dic_tab_position[
                self.RAMSTK_USER_CONFIGURATION.RAMSTK_TABPOS[book].lower()
            ]
        except KeyError:
            _tab_position = self.dic_tab_position["bottom"]
        self.set_tab_pos(_tab_position)



# Standard Library Imports
from typing import Any

# Third Party Imports
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKRevisionRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any
    __tablename__: str
    __table_args__: Any
    revision_id: Any
    availability_logistics: Any
    availability_mission: Any
    cost: Any
    cost_failure: Any
    cost_hour: Any
    hazard_rate_active: Any
    hazard_rate_dormant: Any
    hazard_rate_logistics: Any
    hazard_rate_mission: Any
    hazard_rate_software: Any
    mmt: Any
    mcmt: Any
    mpmt: Any
    mtbf_logistics: Any
    mtbf_mission: Any
    mttr: Any
    name: Any
    reliability_logistics: Any
    reliability_mission: Any
    remarks: Any
    total_part_count: Any
    revision_code: Any
    program_time: Any
    program_time_sd: Any
    program_cost: Any
    program_cost_sd: Any
    failures: relationship
    mission: relationship
    function: relationship
    requirement: relationship
    stakeholder: relationship
    hardware: relationship
    validation: relationship
    hazard: relationship
    program_status: relationship
    def get_attributes(self): ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.allocation.panel.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Allocation Panels."""

# Standard Library Imports
from typing import Any, Dict, List, Union

# Third Party Imports
import treelib
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import (
    RAMSTKComboBox,
    RAMSTKEntry,
    RAMSTKFixedPanel,
    RAMSTKTreePanel,
)


class AllocationGoalMethodPanel(RAMSTKFixedPanel):
    """Panel to display reliability Allocation goals and method."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field = "hardware_id"
    _select_msg = "selected_allocation"
    _tag = "allocation"
    _title = _("Allocation Goals and Method")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self):
        """Initialize an instance of the Allocation goals and method panel."""
        super().__init__()

        # Initialize widgets.
        self.cmbAllocationGoal: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbAllocationMethod: RAMSTKComboBox = RAMSTKComboBox()
        self.txtHazardRateGoal: RAMSTKEntry = RAMSTKEntry()
        self.txtMTBFGoal: RAMSTKEntry = RAMSTKEntry()
        self.txtReliabilityGoal: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary instance attributes.

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.
        self._goal_id: int = 0
        self._on_edit_message = f"wvw_editing_{self._tag}"

        # Initialize public dictionary instance attributes.
        self.dic_attribute_widget_map = {
            "allocation_method_id": [
                10,
                self.cmbAllocationMethod,
                "changed",
                super().on_changed_combo,
                self._on_edit_message,
                0,
                {
                    "tooltip": _(
                        "Selects the method for allocating the reliability goal for "
                        "the selected hardware assembly."
                    ),
                },
                _("Select Allocation Method:"),
                "gint",
            ],
            "goal_measure_id": [
                5,
                self.cmbAllocationGoal,
                "changed",
                super().on_changed_combo,
                self._on_edit_message,
                0,
                {
                    "tooltip": _(
                        "Selects the goal measure for the selected hardware assembly."
                    ),
                },
                _("Select Goal Metric:"),
                "gint",
            ],
            "reliability_goal": [
                19,
                self.txtReliabilityGoal,
                "changed",
                super().on_changed_entry,
                self._on_edit_message,
                1.0,
                {
                    "tooltip": _(
                        "Displays the reliability goal for the selected hardware item."
                    ),
                    "width": 125,
                },
                _("R(t) Goal:"),
                "gfloat",
            ],
            "hazard_rate_goal": [
                7,
                self.txtHazardRateGoal,
                "changed",
                super().on_changed_entry,
                self._on_edit_message,
                0.0,
                {
                    "tooltip": _(
                        "Displays the hazard rate goal for the selected hardware item."
                    ),
                    "width": 125,
                },
                _("h(t) Goal:"),
                "gfloat",
            ],
            "mtbf_goal": [
                13,
                self.txtMTBFGoal,
                "changed",
                super().on_changed_entry,
                self._on_edit_message,
                0.0,
                {
                    "tooltip": _(
                        "Displays the MTBF goal for the selected hardware item."
                    ),
                    "width": 125,
                },
                _("MTBF Goal:"),
                "gfloat",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.cmbAllocationMethod.connect("changed", self._on_method_changed)
        self.cmbAllocationGoal.connect("changed", self._on_goal_changed)

        # Subscribe to PyPubSub messages.

    def do_load_comboboxes(self) -> None:
        """Load the RAMSTKComboBox() widgets.

        :return: None
        :rtype: None
        """
        self.cmbAllocationGoal.do_load_combo(
            [[_("Reliability"), 0], [_("Hazard Rate"), 1], [_("MTBF"), 2]]
        )
        self.cmbAllocationMethod.do_load_combo(
            [
                [_("Equal Apportionment"), 0],
                [_("AGREE Apportionment"), 1],
                [_("ARINC Apportionment"), 2],
                [_("Feasibility of Objectives"), 3],
            ]
        )

    def _do_set_sensitive(self, attributes) -> None:
        """Set widget sensitivity as needed for the selected R(t) goal.

        :return: None
        :rtype: None
        """
        self.cmbAllocationGoal.set_sensitive(True)
        self.cmbAllocationGoal.do_update(
            attributes["goal_measure_id"], signal="changed"
        )
        self.cmbAllocationMethod.set_sensitive(True)
        self.cmbAllocationMethod.do_update(
            attributes["allocation_method_id"], signal="changed"
        )
        self.txtReliabilityGoal.set_sensitive(False)
        self.txtMTBFGoal.set_sensitive(False)
        self.txtHazardRateGoal.set_sensitive(False)

        if self._goal_id == 1:  # Expressed as reliability.
            self.txtReliabilityGoal.set_sensitive(True)
            self.txtReliabilityGoal.do_update(
                attributes["reliability_goal"],
                signal="changed",
            )
        elif self._goal_id == 2:  # Expressed as a hazard rate.
            self.txtHazardRateGoal.set_sensitive(True)
            self.txtHazardRateGoal.do_update(
                attributes["hazard_rate_goal"],
                signal="changed",
            )
        elif self._goal_id == 3:  # Expressed as an MTBF.
            self.txtMTBFGoal.set_sensitive(True)
            self.txtMTBFGoal.do_update(
                attributes["mtbf_goal"],
                signal="changed",
            )

    def _on_goal_changed(self, combo: RAMSTKComboBox) -> None:
        """Let others know when allocation goal combo changes.

        :param combo: the allocation goal type RAMSTKComboBox().
        :return: None
        :rtype: None
        """
        self._goal_id = combo.get_active()

    @staticmethod
    def _on_method_changed(combo: RAMSTKComboBox) -> None:
        """Let others know when allocation method combo changes.

        :param combo: the allocation calculation method RAMSTKComboBox().
        :return: None
        :rtype: None
        """
        pub.sendMessage(
            "succeed_change_allocation_method", method_id=combo.get_active()
        )


class AllocationTreePanel(RAMSTKTreePanel):
    """Panel to display reliability Allocation worksheet."""

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _select_msg = "succeed_retrieve_allocations"
    _tag = "allocation"
    _title = _("Allocation Analysis")

    # Define public dictionary class attributes.

    # Define public dictionary list attributes.

    # Define public dictionary scalar attributes.

    def __init__(self):
        """Initialize an instance of the Allocation worksheet panel."""
        super().__init__()

        # Initialize private dictionary attributes.
        self._dic_hardware_attrs: Dict[int, List[Union[bool, float, int, str]]] = {}
        self._dic_reliability_attrs: Dict[int, List[Union[bool, float, int, str]]] = {}
        self.tvwTreeView.dic_row_loader = {
            "allocation": self.__do_load_allocation,
        }
        self._dic_visible_mask: Dict[int, Dict[str, str]] = {
            0: {
                "revision_id": False,
                "hardware_id": False,
                "name": True,
                "included": False,
                "n_sub_systems": False,
                "n_sub_elements": False,
                "mission_time": False,
                "duty_cycle": False,
                "int_factor": False,
                "soa_factor": False,
                "op_time_factor": False,
                "env_factor": False,
                "weight_factor": False,
                "percent_weight_factor": False,
                "hazard_rate_logistics": True,
                "hazard_rate_alloc": False,
                "mtbf_logistics": True,
                "mtbf_alloc": False,
                "reliability_logistics": True,
                "reliability_alloc": False,
                "availability_logistics": True,
                "availability_alloc": False,
            },
            1: {
                "revision_id": False,
                "hardware_id": False,
                "name": True,
                "included": True,
                "n_sub_systems": True,
                "n_sub_elements": False,
                "mission_time": True,
                "duty_cycle": False,
                "int_factor": False,
                "soa_factor": False,
                "op_time_factor": False,
                "env_factor": False,
                "weight_factor": False,
                "percent_weight_factor": False,
                "hazard_rate_logistics": True,
                "hazard_rate_alloc": True,
                "mtbf_logistics": True,
                "mtbf_alloc": True,
                "reliability_logistics": True,
                "reliability_alloc": True,
                "availability_logistics": True,
                "availability_alloc": True,
            },
            2: {
                "revision_id": False,
                "hardware_id": False,
                "name": True,
                "included": True,
                "n_sub_systems": True,
                "n_sub_elements": True,
                "mission_time": True,
                "duty_cycle": True,
                "int_factor": False,
                "soa_factor": False,
                "op_time_factor": False,
                "env_factor": False,
                "weight_factor": True,
                "percent_weight_factor": True,
                "hazard_rate_logistics": True,
                "hazard_rate_alloc": True,
                "mtbf_logistics": True,
                "mtbf_alloc": True,
                "reliability_logistics": True,
                "reliability_alloc": True,
                "availability_logistics": True,
                "availability_alloc": True,
            },
            3: {
                "revision_id": False,
                "hardware_id": False,
                "name": True,
                "included": True,
                "n_sub_systems": False,
                "n_sub_elements": False,
                "mission_time": False,
                "duty_cycle": False,
                "int_factor": False,
                "soa_factor": False,
                "op_time_factor": False,
                "env_factor": False,
                "weight_factor": True,
                "percent_weight_factor": False,
                "hazard_rate_logistics": True,
                "hazard_rate_alloc": True,
                "mtbf_logistics": True,
                "mtbf_alloc": True,
                "reliability_logistics": True,
                "reliability_alloc": True,
                "availability_logistics": True,
                "availability_alloc": True,
            },
            4: {
                "revision_id": False,
                "hardware_id": False,
                "name": True,
                "included": True,
                "n_sub_systems": False,
                "n_sub_elements": False,
                "mission_time": False,
                "duty_cycle": False,
                "int_factor": True,
                "soa_factor": True,
                "op_time_factor": True,
                "env_factor": True,
                "weight_factor": True,
                "percent_weight_factor": False,
                "hazard_rate_logistics": True,
                "hazard_rate_alloc": True,
                "mtbf_logistics": True,
                "mtbf_alloc": True,
                "reliability_logistics": True,
                "reliability_alloc": True,
                "availability_logistics": True,
                "availability_alloc": True,
            },
        }

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._goal_id: int = 0
        self._method_id: int = 0
        self._on_edit_message: str = f"mvw_editing_{self._tag}"

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map = {
            "revision_id": [
                0,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Revision ID"),
                "gint",
            ],
            "hardware_id": [
                1,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Hardware ID"),
                "gint",
            ],
            "name": [
                2,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Assembly"),
                "gchararray",
            ],
            "included": [
                3,
                Gtk.CellRendererToggle(),
                "toggled",
                super().on_cell_toggled,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Included?"),
                "gint",
            ],
            "n_sub_systems": [
                4,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Number of Sub-Systems"),
                "gint",
            ],
            "n_sub_elements": [
                5,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Number of Sub-Elements"),
                "gint",
            ],
            "mission_time": [
                6,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Operating Time"),
                "gfloat",
            ],
            "duty_cycle": [
                7,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Duty Cycle"),
                "gfloat",
            ],
            "int_factor": [
                8,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Intricacy (1-10)"),
                "gint",
            ],
            "soa_factor": [
                9,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("State of the Art (1-10)"),
                "gint",
            ],
            "op_time_factor": [
                10,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Operating Time (1-10)"),
                "gint",
            ],
            "env_factor": [
                11,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Environment (1-10)"),
                "gint",
            ],
            "weight_factor": [
                12,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Weighting Factor"),
                "gfloat",
            ],
            "percent_weight_factor": [
                13,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Percent Weighting Factor"),
                "gfloat",
            ],
            "hazard_rate_logistics": [
                14,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Current Hazard Rate"),
                "gfloat",
            ],
            "hazard_rate_alloc": [
                15,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Allocated Failure Rate"),
                "gfloat",
            ],
            "mtbf_logistics": [
                16,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Current MTBF"),
                "gfloat",
            ],
            "mtbf_alloc": [
                17,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Allocation MTBF"),
                "gfloat",
            ],
            "reliability_logistics": [
                18,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Current Reliability"),
                "gfloat",
            ],
            "reliability_alloc": [
                19,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Allocated Reliability"),
                "gfloat",
            ],
            "availability_logistics": [
                20,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Current Availability"),
                "gfloat",
            ],
            "availability_alloc": [
                21,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Allocation Availability"),
                "gfloat",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.tvwTreeView.set_tooltip_text(
            _(
                "Displays the Allocation Analysis for the currently selected "
                "Hardware item."
            )
        )

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_load_panel, "succeed_calculate_allocation")
        pub.subscribe(self._do_set_hardware_attributes, "succeed_retrieve_hardwares")
        pub.subscribe(
            self._do_set_reliability_attributes, "succeed_retrieve_reliabilitys"
        )
        pub.subscribe(self._on_method_changed, "succeed_change_allocation_method")

    def _do_set_columns_visible(self) -> None:
        """Set editable columns based on the Allocation method selected.

        :return: None
        :rtype: None
        """
        self.tvwTreeView.visible = self._dic_visible_mask[self._method_id]
        self.tvwTreeView.do_set_visible_columns()

    def _do_set_hardware_attributes(self, tree: treelib.Tree) -> None:
        """Set the attributes when the hardware tree is retrieved.

        :param tree: the hardware treelib.Tree().
        :return: None
        :rtype: None
        """
        for _node in tree.all_nodes()[1:]:
            _hardware = _node.data["hardware"]
            self._dic_hardware_attrs[_hardware.hardware_id] = [
                _hardware.name,
            ]

    def _do_set_reliability_attributes(self, tree: treelib.Tree) -> None:
        """Set the attributes when the reliability tree is retrieved.

        :param tree: the reliability treelib.Tree().
        :return: None
        :rtype: None
        """
        for _node in tree.all_nodes()[1:]:
            _reliability = _node.data["reliability"]
            self._dic_reliability_attrs[_reliability.hardware_id] = [
                _reliability.hazard_rate_logistics,
                _reliability.mtbf_logistics,
                _reliability.reliability_logistics,
                _reliability.availability_logistics,
            ]

    def _on_method_changed(self, method_id: int) -> None:
        """Set method ID attributes when user changes the selection.

        :param method_id: the newly selected allocation method.
        :return: None
        :rtype: None
        """
        self._method_id = method_id
        self._do_set_columns_visible()

    def _on_row_change(self, selection: Gtk.TreeSelection) -> None:
        """Handle events for the Hardware package Module View RAMSTKTreeView().

        This method is called whenever a Hardware Module View RAMSTKTreeView()
        row is activated/changed.

        :param selection: the Hardware class Gtk.TreeSelection().
        :return: None
        """
        _attributes = super().on_row_change(selection)

        if _attributes:
            self._record_id = _attributes["hardware_id"]

            pub.sendMessage(
                "selected_allocation",
                attributes=_attributes,
            )
            pub.sendMessage(
                "request_get_allocation_attributes",
                node_id=self._record_id,
            )

    def __do_load_allocation(
        self, node: Any = "", row: Gtk.TreeIter = None
    ) -> Gtk.TreeIter:
        """Load the allocation RAMSTKTreeView().

        :param node: the treelib Node() with the mode data to load.
        :param row: the parent row of the mode to load into the hardware tree.
        :return: _new_row; the row that was just populated with hardware data.
        :rtype: :class:`Gtk.TreeIter`
        """
        _new_row = None

        _entity = node.data["allocation"]

        if not _entity.parent_id == 0:
            _model = self.tvwTreeView.get_model()

            try:
                _name = self._dic_hardware_attrs[_entity.hardware_id][0]
                _hr_logistics = self._dic_reliability_attrs[_entity.hardware_id][0]
                _mtbf_logistics = self._dic_reliability_attrs[_entity.hardware_id][1]
                _rel_logistics = self._dic_reliability_attrs[_entity.hardware_id][2]
                _avail_logistics = self._dic_reliability_attrs[_entity.hardware_id][3]
            except KeyError:
                _name = ""
                _hr_logistics = 0.0
                _mtbf_logistics = 0.0
                _rel_logistics = 1.0
                _avail_logistics = 1.0

            _attributes = [
                _entity.revision_id,
                _entity.hardware_id,
                _name,
                _entity.included,
                _entity.n_sub_systems,
                _entity.n_sub_elements,
                _entity.mission_time,
                _entity.duty_cycle,
                _entity.int_factor,
                _entity.soa_factor,
                _entity.op_time_factor,
                _entity.env_factor,
                _entity.weight_factor,
                _entity.percent_weight_factor,
                _hr_logistics,
                _entity.hazard_rate_alloc,
                _mtbf_logistics,
                _entity.mtbf_alloc,
                _rel_logistics,
                _entity.reliability_alloc,
                _avail_logistics,
                _entity.availability_alloc,
            ]

            try:
                _new_row = _model.append(row, _attributes)
            except (AttributeError, TypeError, ValueError):
                _new_row = None
                _message = _(
                    f"An error occurred when loading allocation record "
                    f"{str(node.identifier)} into the allocation list.  This might "
                    f"indicate it was missing it's data package, some of the data in "
                    f"the package was missing, or some of the data was the wrong "
                    f"type.  Row data was: {_attributes}"
                )
                pub.sendMessage(
                    "do_log_warning_msg",
                    logger_name="WARNING",
                    message=_message,
                )

        return _new_row



# Standard Library Imports
from typing import Any, Dict

PART_COUNT_LAMBDA_B: Any
PART_COUNT_PI_Q: Any
PART_STRESS_PI_Q: Any
PI_C: Any
PI_CF: Any
PI_E: Any
REF_TEMPS: Any

def calculate_capacitance_factor(subcategory_id: int, capacitance: float) -> float: ...
def calculate_part_count(**attributes: Dict[str, Any]) -> float: ...
def calculate_part_stress(**attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def calculate_part_stress_lambda_b(
    subcategory_id: int,
    temperature_rated_max: float,
    temperature_active: float,
    voltage_ratio: float,
) -> float: ...
def calculate_series_resistance_factor(
    resistance: float, voltage_dc_operating: float, voltage_ac_operating: float
) -> float: ...
def get_configuration_factor(configuration_id: int) -> float: ...
def get_construction_factor(construction_id: int) -> float: ...
def get_part_count_lambda_b(
    subcategory_id: int, environment_active_id: int, specification_id: int = ...
) -> float: ...



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKSiteInfoRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any
    __tablename__: str
    __table_args__: Any
    site_id: Any
    site_name: Any
    product_key: Any
    expire_on: Any
    function_enabled: Any
    requirement_enabled: Any
    hardware_enabled: Any
    software_enabled: Any
    rcm_enabled: Any
    testing_enabled: Any
    incident_enabled: Any
    survival_enabled: Any
    vandv_enabled: Any
    hazard_enabled: Any
    stakeholder_enabled: Any
    allocation_enabled: Any
    similar_item_enabled: Any
    fmea_enabled: Any
    pof_enabled: Any
    rbd_enabled: Any
    fta_enabled: Any
    def get_attributes(self): ...



# pylint: disable=non-parent-init-called
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.widgets.combo.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTK GTK3 Combo Module."""

# Standard Library Imports
from typing import Any, Dict, List, Union

# RAMSTK Package Imports
from ramstk.utilities import none_to_default
from ramstk.views.gtk3 import GObject, Gtk

# RAMSTK Local Imports
from .widget import RAMSTKWidget


class RAMSTKComboBox(Gtk.ComboBox, RAMSTKWidget):
    """The RAMSTK ComboBox class."""

    # Define private class scalar attributes.
    _default_height = 30
    _default_width = 200

    def __init__(self, index: int = 0, simple: bool = True) -> None:
        """Create RAMSTK ComboBox widgets.

        :keyword int index: the index in the RAMSTKComboBox Gtk.ListView() to
            display.  Default is 0.
        :keyword bool simple: indicates whether to make a simple (one item) or
            complex (three item) RAMSTKComboBox.  Default is True.
        """
        RAMSTKWidget.__init__(self)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._index: int = index

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        if not simple:
            _list = Gtk.ListStore()
            _list.set_column_types(
                [GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_STRING]
            )
        else:
            _list = Gtk.ListStore()
            _list.set_column_types([GObject.TYPE_STRING])
        self.set_model(_list)

        _cell = Gtk.CellRendererText()
        self.pack_start(_cell, True)
        self.add_attribute(_cell, "text", self._index)

        self.show()

    def do_get_options(self) -> Dict[int, Any]:
        """Retrieve all the options in the RAMSTK Combo.

        :return: _options
        :rtype: dict
        """
        _options = {}

        _model = self.get_model()
        _iter = _model.get_iter_first()

        i = 0
        while _iter is not None:
            _options[i] = _model.get_value(_iter, self._index)
            _iter = _model.iter_next(_iter)
            i += 1

        return _options

    # noinspection PyIncorrectDocstring
    def do_load_combo(
        self,
        entries: List[List[Union[str, int]]],
        signal: str = "",
        simple: bool = True,
    ) -> None:
        """Load RAMSTK ComboBox widgets.

        :param entries: the information to load into the Gtk.ComboBox().
            This is always a list of lists where each internal list contains
            the information to be displayed and there is one internal list for
            each RAMSTKComboBox line.
        :param signal: the name of the signal whose handler ID the
            RAMSTKComboBox() needs to block.
        :param simple: indicates whether this is a simple (one item) or
            complex (three item) RAMSTKComboBox.  A simple (default)
            RAMSTKComboBox contains and displays one field only.  A 'complex'
            RAMSTKComboBox contains three str fields, but only displays the
            first field.  The other two fields are hidden and used to store
            information associated with the items displayed in the
            RAMSTKComboBox.  For example, if the name of an item is displayed,
            the other two fields might contain a code and an index.  These
            could be extracted for use in the RAMSTK Views.
        :return: None
        :raise: TypeError if attempting to load other than string values.
        """
        _model = self.get_model()
        _model.clear()

        try:
            _handler_id = self.dic_handler_id[signal]
            self.handler_block(_handler_id)
        except KeyError:
            _handler_id = -1

        if not simple:
            _model.append(["", "", ""])
            # pylint: disable=unused-variable
            for __, _entry in enumerate(entries):
                _model.append(list(_entry))
        else:
            _model.append([""])
            # pylint: disable=unused-variable
            for __, _entry in enumerate(entries):
                _model.append([_entry[self._index]])

        if _handler_id > 0:
            self.handler_unblock(_handler_id)

    def do_update(self, value: int, signal: str = "") -> None:
        """Update the RAMSTK Combo with a new value.

        :param value: the information to update the RAMSTKCombo() to
            display.
        :param str signal: the name of the signal whose handler ID the
            RAMSTKComboBox() needs to block.
        :return: None
        :rtype: None
        """
        _handler_id = self.dic_handler_id[signal]

        _value = none_to_default(value, 0)

        self.handler_block(_handler_id)
        self.set_active(_value)
        self.handler_unblock(_handler_id)

    def get_value(self, index: int = 0) -> str:
        """Return value in the RAMSTKComboBox() model at the index position.

        :keyword int index: the column in the RAMSTKComboBox() model whose
            value is to be retrieved.  Defaults to zero which will always
            read a 'simple' RAMSTKComboBox().
        :return: _value
        :rtype: str
        """
        _model = self.get_model()
        _row = self.get_active_iter()

        _value: str = _model.get_value(_row, index)

        return _value



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKOpStressRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any
    __tablename__: str
    __table_args__: Any
    revision_id: Any
    hardware_id: Any
    mode_id: Any
    mechanism_id: Any
    load_id: Any
    stress_id: Any
    description: Any
    load_history: Any
    measurable_parameter: Any
    remarks: Any
    op_load: Any
    is_mode: bool
    is_mechanism: bool
    is_opload: bool
    is_opstress: bool
    is_testmethod: bool
    def get_attributes(self): ...



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.environment.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Environment Record Model."""

# Third Party Imports
from sqlalchemy import Column, Float, ForeignKeyConstraint, Integer, String
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKEnvironmentRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_environment table in RAMSTK Program database.

    This table shares a Many-to-One relationship with ramstk_mission_phase.
    """

    __defaults__ = {
        "name": "Condition Name",
        "units": "Units",
        "minimum": 0.0,
        "maximum": 0.0,
        "mean": 0.0,
        "variance": 0.0,
        "ramp_rate": 0.0,
        "low_dwell_time": 0.0,
        "high_dwell_time": 0.0,
    }
    __tablename__ = "ramstk_environment"
    __table_args__ = (
        ForeignKeyConstraint(
            ["fld_revision_id", "fld_mission_id", "fld_phase_id"],
            [
                "ramstk_mission_phase.fld_revision_id",
                "ramstk_mission_phase.fld_mission_id",
                "ramstk_mission_phase.fld_phase_id",
            ],
        ),
        {"extend_existing": True},
    )

    revision_id = Column(
        "fld_revision_id",
        Integer,
        primary_key=True,
        nullable=False,
    )
    mission_id = Column("fld_mission_id", Integer, primary_key=True, nullable=False)
    phase_id = Column("fld_phase_id", Integer, primary_key=True, nullable=False)
    environment_id = Column(
        "fld_environment_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )

    name = Column("fld_name", String(256), default=__defaults__["name"])
    units = Column("fld_units", String(128), default=__defaults__["units"])
    minimum = Column("fld_minimum", Float, default=__defaults__["minimum"])
    maximum = Column("fld_maximum", Float, default=__defaults__["maximum"])
    mean = Column("fld_mean", Float, default=__defaults__["mean"])
    variance = Column("fld_variance", Float, default=__defaults__["variance"])
    ramp_rate = Column("fld_ramp_rate", Float, default=__defaults__["ramp_rate"])
    low_dwell_time = Column(
        "fld_low_dwell_time", Float, default=__defaults__["low_dwell_time"]
    )
    high_dwell_time = Column(
        "fld_high_dwell_time", Float, default=__defaults__["high_dwell_time"]
    )

    # Define the relationships to other tables in the RAMSTK Program database.
    phase: relationship = relationship(  # type: ignore
        "RAMSTKMissionPhaseRecord",
        back_populates="environment",
    )

    is_mission = False
    is_phase = False
    is_env = True

    def get_attributes(self):
        """Retrieve current values of RAMSTKEnvironment data model attributes.

        :return: {phase_id, environment_id, name, units, minimum,
                  maximum, mean, variance, ramp_rate, low_dwell_time,
                  high_dwell_time} pairs.
        :rtype: dict
        """
        _attributes = {
            "phase_id": self.phase_id,
            "environment_id": self.environment_id,
            "name": self.name,
            "units": self.units,
            "minimum": self.minimum,
            "maximum": self.maximum,
            "mean": self.mean,
            "variance": self.variance,
            "ramp_rate": self.ramp_rate,
            "low_dwell_time": self.low_dwell_time,
            "high_dwell_time": self.high_dwell_time,
        }

        return _attributes



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKCategory.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKCategory Table Module."""

# Third Party Imports
from sqlalchemy import Column, Float, Integer, String
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKCategory(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent table ramstk_category in the RAMSTK Common database.

    Types of category are:     # 1. Hardware     # 2. Risk     # 3.
    Software     # 4. Incident     # 5. Action
    """

    __defaults__ = {
        "name": "Category Name",
        "description": "Category Description",
        "category_type": "unknown",
        "value": 1,
        "harsh_ir_limit": 0.8,
        "mild_ir_limit": 0.9,
        "harsh_pr_limit": 1.0,
        "mild_pr_limit": 1.0,
        "harsh_vr_limit": 1.0,
        "mild_vr_limit": 1.0,
        "harsh_deltat_limit": 0.0,
        "mild_deltat_limit": 0.0,
        "harsh_maxt_limit": 125.0,
        "mild_maxt_limit": 125.0,
    }
    __tablename__ = "ramstk_category"
    __table_args__ = {"extend_existing": True}

    category_id = Column(
        "fld_category_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    name = Column("fld_name", String(256), default=__defaults__["name"])
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    category_type = Column(
        "fld_category_type", String(256), default=__defaults__["category_type"]
    )
    value = Column("fld_value", Integer, default=__defaults__["value"])
    harsh_ir_limit = Column(
        "fld_harsh_ir_limit", Float, default=__defaults__["harsh_ir_limit"]
    )
    mild_ir_limit = Column(
        "fld_mild_ir_limit", Float, default=__defaults__["mild_ir_limit"]
    )
    harsh_pr_limit = Column(
        "fld_harsh_pr_limit", Float, default=__defaults__["harsh_pr_limit"]
    )
    mild_pr_limit = Column(
        "fld_mild_pr_limit", Float, default=__defaults__["mild_pr_limit"]
    )
    harsh_vr_limit = Column(
        "fld_harsh_vr_limit", Float, default=__defaults__["harsh_vr_limit"]
    )
    mild_vr_limit = Column(
        "fld_mild_vr_limit", Float, default=__defaults__["mild_vr_limit"]
    )
    harsh_deltat_limit = Column(
        "fld_harsh_deltat_limit", Float, default=__defaults__["harsh_deltat_limit"]
    )
    mild_deltat_limit = Column(
        "fld_mild_deltat_limit", Float, default=__defaults__["mild_deltat_limit"]
    )
    harsh_maxt_limit = Column(
        "fld_harsh_maxt_limit", Float, default=__defaults__["harsh_maxt_limit"]
    )
    mild_maxt_limit = Column(
        "fld_mild_maxt_limit", Float, default=__defaults__["mild_maxt_limit"]
    )

    # Define the relationships to other tables in the RAMSTK Program database.
    subcategory = relationship(  # type: ignore
        "RAMSTKSubCategory",
        back_populates="category",
        cascade="delete",
    )
    mode = relationship(  # type: ignore
        "RAMSTKFailureMode",
        back_populates="category",
        cascade="delete",
    )

    def get_attributes(self):
        """Retrieve current values of the RAMSTKCategory data model attributes.

        :return: {category_id, name, description, category_type, value,
                  harsh_ir_limit, mild_ir_limit, harsh_pr_limit,
                  mild_pr_limit, harsh_vr_limit, mild_vr_limit,
                  harsh_deltat_limit, mild_deltat_limit, harsh_maxt_limit,
                  mild_maxt_limit} pairs
        :rtype: dict
        """
        _attributes = {
            "category_id": self.category_id,
            "name": self.name,
            "description": self.description,
            "category_type": self.category_type,
            "value": self.value,
            "harsh_ir_limit": self.harsh_ir_limit,
            "mild_ir_limit": self.mild_ir_limit,
            "harsh_pr_limit": self.harsh_pr_limit,
            "mild_pr_limit": self.mild_pr_limit,
            "harsh_vr_limit": self.harsh_vr_limit,
            "mild_vr_limit": self.mild_vr_limit,
            "harsh_deltat_limit": self.harsh_deltat_limit,
            "mild_deltat_limit": self.mild_deltat_limit,
            "harsh_maxt_limit": self.harsh_maxt_limit,
            "mild_maxt_limit": self.mild_maxt_limit,
        }

        return _attributes



# -*- coding: utf-8 -*-
#
#       ramstk.analyses.Stress.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Component Stress Calculations Module."""


def calculate_stress_ratio(stress_operating, stress_rated):
    """Calculate the operating electrical stress ratio of a device.

    Inputs can be floats, integers, or a combination.

    >>> calculate_stress_ratio(0.382, 1.29)
    0.2961240310077519
    >>> calculate_stress_ratio(1, 2)
    0.5
    >>> calculate_stress_ratio(0.382, 2)
    0.191
    >>> calculate_stress_ratio(1, 1.29)
    0.7751937984496123

    Rated stress must not be zero:
    >>> calculate_stress_ratio(0.382, 0.0)
    Traceback (most recent call last):
        ...
    ZeroDivisionError: float division by zero

    Stress inputs must not be strings:
    >>>  calculate_stress_ratio(0.382, '3.2')
    Traceback (most recent call last):
        ...
    TypeError: unsupported operand type(s) for /: 'float' and 'str'

    :param stress_operating: the device's operating level of the stress.
    :param stress_rated: the devices's rated stress.
    :return: _stress_ratio; the ratio of operating stress to rated stress.
    :rtype: float
    :raise: TypeError if an input value is non-numerical.
    :raise: ZeroDivisionError if the rated stress is zero.
    """
    return stress_operating / stress_rated



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.assistants.imports.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2021 Doyle "weibullguy" Rowland
"""Import Assistant Module."""

# Standard Library Imports
import os
from typing import List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.views.gtk3 import GObject, Gtk, _
from ramstk.views.gtk3.widgets import RAMSTKComboBox, RAMSTKLabel


class ImportProject(Gtk.Assistant):
    """Assistant to walk user through the process of importing records."""

    RAMSTK_USER_CONFIGURATION = RAMSTKUserConfiguration()

    def __init__(
        self,
        __button: Gtk.ToolButton,
        configuration: RAMSTKUserConfiguration,
        parent: object,
    ):
        """Initialize an instance of the Import Assistant.

        :param __button: the Gtk.ToolButton() that launched an instance of this
            class.
        :param configuration: the RAMSTKUserConfiguration class instance.
        :param parent: the parent window to associate this dialog with.
        """
        super().__init__()
        self.set_transient_for(parent)

        # Initialize private dict variables.

        # Initialize private list variables.

        # Initialize private scalar variables.
        self.cmbSelectModule: RAMSTKComboBox = RAMSTKComboBox()
        self._filechooser: Gtk.FileChooserButton = Gtk.FileChooserButton(
            action=Gtk.FileChooserAction.OPEN
        )
        self._module: str = ""
        self.tvwFieldMap: Gtk.TreeView = Gtk.TreeView()

        # Initialize public dict variables.

        # Initialize public list variables.

        # Initialize public scalar variables.
        self.RAMSTK_USER_CONFIGURATION = configuration

        self.__do_set_properties()
        self.__make_ui()
        self.__do_load_combobox()
        self.__do_set_callbacks()

        pub.subscribe(self._do_load_db_fields, "succeed_read_db_fields")
        pub.subscribe(self._do_load_import_fields, "succeed_read_import_file")

    def _do_edit_cell(
        self, __cell: Gtk.CellRenderer, path: str, new_text: str, model: Gtk.TreeModel
    ) -> None:
        """Handle Gtk.CellRenderer() edits.

        :param __cell: the Gtk.CellRenderer() that was edited.
        :param path: the Gtk.TreeView() path of the Gtk.CellRenderer() that
                         was edited.
        :param new_text: the new text in the edited Gtk.CellRenderer().
        :param model: the Gtk.TreeModel() the Gtk.CellRenderer() belongs to.
        :return: None
        :rtype: None
        """
        _page = self.get_nth_page(2)

        _db_field = model[path][0]
        model[path][1] = new_text

        pub.sendMessage(
            "request_map_to_field",
            module=self._module,
            import_field=new_text,
            format_field=_db_field,
        )

        self.set_page_complete(_page, True)

    def _do_load_db_fields(self, db_fields: List[str]) -> None:
        """Load the Rosetta stone with the names of database fields.

        :param list db_fields: the list of database field names.
        :return: None
        :rtype: None
        """
        _model = self.tvwFieldMap.get_model()
        _model.clear()

        for _field in db_fields:
            _model.append(None, [_field, ""])

    def _do_load_import_fields(self, import_fields: List[str]) -> None:
        """Load the Rosetta stone with the import file field names.

        :param list import_fields: the list of field names found in the
            import file.
        :return: None
        :rtype: None
        """
        _cell = self.tvwFieldMap.get_column(1).get_cells()[0]
        _cellmodel = _cell.get_property("model")
        _cellmodel.clear()
        _cellmodel.append([""])

        # pylint: disable=unused-variable
        for __, _field in enumerate(import_fields):
            _cellmodel.append([_field])

    def _do_quit(self, __widget: Gtk.Widget) -> None:
        """Quit the RAMSTK Import Assistant.

        :param __widget: the Gtk.Widget() that called this method.
        :return: None
        :rtype: None
        """
        self.destroy()

    def _do_request_import(self, __assistant: Gtk.Assistant) -> None:
        """Request the data controller import new records.

        :param __assistant: this Gtk.Assistant() instance.
        :return: None
        :rtype: None
        """
        pub.sendMessage(
            "request_import",
            module=self._module,
        )

    def _do_select_file(self, filechooser: Gtk.FileChooser) -> None:
        """Select the input file to be read.

        :param filechooser: the Gtk.FileChooser() that called this method.
        :return: None
        :rtype: None
        """
        _file_type = ""
        _page = self.get_nth_page(1)

        _file = filechooser.get_filename()
        # pylint: disable=unused-variable
        __, _extension = os.path.splitext(_file)

        if _extension == ".csv":
            _file_type = "csv"
        elif _extension == ".txt":
            _file_type = "text"
        elif _extension in [".xls", ".xlsx", ".xlsm"]:
            _file_type = "excel"

        if _file is not None:
            pub.sendMessage(
                "request_read_import_file", file_type=_file_type, file_name=_file
            )

        self.set_page_complete(_page, True)

    def _on_combo_changed(self, combo: RAMSTKComboBox) -> None:
        """Respond to Gtk.ComboBox() 'changed' signals.

        :param Gtk.ComboBox combo: the Gtk.ComboBox() that called this method.
        :return: None
        :rtype: None
        """
        self._module = combo.get_value()

        pub.sendMessage(
            "request_db_fields",
            module=self._module,
        )

    def __do_load_combobox(self) -> None:
        """Load the RAMSTKComboBox() widgets.

        :return: None
        :rtype: None
        """
        self.cmbSelectModule.do_load_combo(
            [
                [""],
                [_("Function")],
                [_("Requirement")],
                [_("Hardware")],
                [_("Validation")],
            ]
        )
        self.cmbSelectModule.set_active(1)

    def __do_set_callbacks(self) -> None:
        """Set the callback methods and functions.

        :return: None
        :rtype: None
        """
        self.connect("cancel", self._do_quit)
        self.connect("close", self._do_quit)
        self.connect("apply", self._do_request_import)

        # ----- COMBOBOX
        self.cmbSelectModule.dic_handler_id["changed"] = self.cmbSelectModule.connect(
            "changed", self._on_combo_changed
        )

        # ----- FILECHOOSER
        self._filechooser.connect("selection_changed", self._do_select_file)

    def __do_set_properties(self) -> None:
        """Set the properties of the Import Assistant and widgets.

        :return: None
        :rtype: None
        """
        self.set_property("title", _("RAMSTK Import Assistant"))
        self.resize(800, 400)
        self.move(200, 100)

        # ----- FILECHOOSER
        _filefilter = Gtk.FileFilter()
        _filefilter.set_name(_("Delimited Text Files"))
        _filefilter.add_pattern("*.csv")
        _filefilter.add_pattern("*.txt")
        self._filechooser.add_filter(_filefilter)

        _filefilter = Gtk.FileFilter()
        _filefilter.set_name(_("Excel Files"))
        _filefilter.add_pattern("*.xls*")
        self._filechooser.add_filter(_filefilter)
        self._filechooser.set_current_folder(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_PROG_DIR
        )

    def __make_confirm_page(self):
        """Make the Import Assistant confirmation page.

        :return: _page
        :rtype: :class:`Gtk.Fixed`
        """
        _page: Gtk.Fixed = Gtk.Fixed()

        _label: RAMSTKLabel = RAMSTKLabel(
            _(
                "RAMSTK is all set and ready to import your "
                "data.  Press 'Apply' to import or 'Cancel' "
                "to abort the import."
            )
        )
        _label.do_set_properties(height=300, width=400, wrap=True)

        _page.put(_label, 25, 5)
        self.append_page(_page)
        self.set_page_type(_page, Gtk.AssistantPageType.CONFIRM)
        self.set_page_complete(_page, True)

    def __make_input_file_select_page(self) -> None:
        """Make the Import Assistant page to select the input file.

        :return: _page
        :rtype: :class:`Gtk.ScrolledWindow`
        """
        _page: Gtk.Fixed = Gtk.Fixed()

        _label = RAMSTKLabel(_("Select the RAMSTK module to import:"))
        _page.put(_label, 5, 5)
        _page.put(self.cmbSelectModule, _label.get_attribute("width") + 10, 5)

        _page.put(self._filechooser, _label.get_attribute("width") + 10, 65)

        self.append_page(_page)
        self.set_page_type(_page, Gtk.AssistantPageType.CONTENT)
        self.set_page_title(_page, _("Select Input File"))

    def __make_introduction_page(self) -> None:
        """Make the Import Assistant introduction page.

        :return: _page
        :rtype: :class:`Gtk.Fixed`
        """
        _page: Gtk.Fixed = Gtk.Fixed()

        _label = RAMSTKLabel(
            _(
                "This is the RAMSTK Import Assistant.  It "
                "will guide you through the process of "
                "importing RAMSTK Program module data from "
                "an external file.  Press 'Forward' to continue "
                "or 'Cancel' to quit."
            )
        )
        _label.do_set_properties(height=300, width=400, wrap=True)

        _page.put(_label, 25, 5)

        self.append_page(_page)
        self.set_page_type(_page, Gtk.AssistantPageType.INTRO)
        self.set_page_title(_page, _("RAMSTK Import Assistant"))
        self.set_page_complete(_page, True)

    def __make_map_field_page(self):
        """Make the Import Assistant page to map fields.

        This method allows the user to map input file fields to RAMSTK
        database table fields.

        :return: _page
        :rtype: :class:`Gtk.ScrolledWindow`
        """
        _page = Gtk.ScrolledWindow()
        _page.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
        _page.add(self.tvwFieldMap)

        _model = Gtk.TreeStore(GObject.TYPE_STRING, GObject.TYPE_STRING)

        _column = Gtk.TreeViewColumn()
        _label = RAMSTKLabel(_("Import File Column"))
        _label.do_set_properties(justify=Gtk.Justification.CENTER)
        _column.set_widget(_label)

        _cell = Gtk.CellRendererText()
        _cell.set_property("foreground", "#000000")
        _cell.set_property("cell-background", "light gray")
        _column.pack_start(_cell, True)
        _column.set_attributes(_cell, text=0)
        _column.set_visible(True)
        self.tvwFieldMap.append_column(_column)

        _column = Gtk.TreeViewColumn()
        _label = RAMSTKLabel(_("RAMSTK Field"))
        _label.do_set_properties(justify=Gtk.Justification.CENTER)
        _column.set_widget(_label)

        _cell = Gtk.CellRendererCombo()
        _cellmodel = Gtk.ListStore(GObject.TYPE_STRING)
        _cellmodel.append([""])
        _cell.set_property("editable", True)
        _cell.set_property("foreground", "#FFFFFF")
        _cell.set_property("has-entry", False)
        _cell.set_property("model", _cellmodel)
        _cell.set_property("text-column", 0)
        _cell.connect("edited", self._do_edit_cell, _model)
        _column.pack_start(_cell, True)
        _column.set_attributes(_cell, text=1)
        _column.set_visible(True)
        self.tvwFieldMap.append_column(_column)

        self.tvwFieldMap.set_model(_model)

        self.append_page(_page)
        self.set_page_type(_page, Gtk.AssistantPageType.CONTENT)
        self.set_page_title(_page, _("Map Input File Fields to RAMSTK Database Fields"))

    def __make_ui(self):
        """Build the user interface.

        :return: None
        :rtype: None
        """
        # Build the assistant.
        self.__make_introduction_page()
        self.__make_input_file_select_page()
        self.__make_map_field_page()
        self.__make_confirm_page()

        self.show_all()



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKFailureMode.py is part of The RAMSTK
#       Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKFailureMode Table Module."""

# Third Party Imports
from sqlalchemy import Column, Float, ForeignKey, Integer, String
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKFailureMode(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_failuremode in the RAMSTK Common database."""

    __defaults__ = {
        "description": "Failure Mode Description",
        "mode_ratio": 1.0,
        "source": "",
    }
    __tablename__ = "ramstk_failure_mode"
    __table_args__ = {"extend_existing": True}

    category_id = Column(
        "fld_category_id",
        Integer,
        ForeignKey("ramstk_category.fld_category_id"),
        nullable=False,
    )
    subcategory_id = Column(
        "fld_subcategory_id",
        Integer,
        ForeignKey("ramstk_subcategory.fld_subcategory_id"),
        nullable=False,
    )
    mode_id = Column(
        "fld_failuremode_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    description = Column(
        "fld_description",
        String(512),
        default=__defaults__["description"],
    )
    mode_ratio = Column("fld_mode_ratio", Float, default=__defaults__["mode_ratio"])
    source = Column("fld_source", String(128), default=__defaults__["source"])

    # Define the relationships to other tables in the RAMSTK Program database.
    category = relationship("RAMSTKCategory", back_populates="mode")  # type: ignore
    subcategory = relationship(  # type: ignore
        "RAMSTKSubCategory", back_populates="mode"
    )

    def get_attributes(self):
        """Retrieve current values of RAMSTKFailureMode data model attributes.

        :return: {failuremode_id, description, type} pairs
        :rtype: dict
        """
        _attributes = {
            "category_id": self.category_id,
            "subcategory_id": self.subcategory_id,
            "mode_id": self.mode_id,
            "description": self.description,
            "mode_ratio": self.mode_ratio,
            "source": self.source,
        }

        return _attributes



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKActionRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any = ...
    __tablename__: str = ...
    __table_args__: Any = ...
    revision_id: Any = ...
    hardware_id: Any = ...
    mode_id: Any = ...
    mechanism_id: Any = ...
    cause_id: Any = ...
    action_id: Any = ...
    action_recommended: Any = ...
    action_category: Any = ...
    action_owner: Any = ...
    action_due_date: Any = ...
    action_status: Any = ...
    action_taken: Any = ...
    action_approved: Any = ...
    action_approve_date: Any = ...
    action_closed: Any = ...
    action_close_date: Any = ...
    cause: Any = ...
    is_mode: bool = ...
    is_mechanism: bool = ...
    is_cause: bool = ...
    is_control: bool = ...
    is_action: bool = ...
    def get_attributes(self): ...



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKStakeholderRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any
    __tablename__: str
    __table_args__: Any
    revision_id: Any
    stakeholder_id: Any
    customer_rank: Any
    description: Any
    group: Any
    improvement: Any
    overall_weight: Any
    planned_rank: Any
    priority: Any
    requirement_id: Any
    stakeholder: Any
    user_float_1: Any
    user_float_2: Any
    user_float_3: Any
    user_float_4: Any
    user_float_5: Any
    def get_attributes(self): ...



# -*- coding: utf-8 -*-
#
#       ramstk.models.basemodel.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Metaclasses for the Record, Table, and View models."""

# Standard Library Imports
import inspect
from typing import Any, Callable, Dict, List, Type

# Third Party Imports
import treelib
from pubsub import pub
from treelib.exceptions import NodeIDAbsentError

# RAMSTK Package Imports
from ramstk.db.base import BaseDatabase
from ramstk.exceptions import DataAccessError
from ramstk.utilities import none_to_default


def do_clear_tree(tree: treelib.Tree) -> treelib.Tree:
    """Clear all nodes from a tree except the root node.

    :param tree: the treelib.Tree to clear.
    :return: the original tree with all child nodes removed.
    :rtype: treelib.Tree
    """
    for _node in tree.children(tree.root):
        tree.remove_node(_node.identifier)

    return tree


class RAMSTKBaseRecord:
    """Meta-class for all RAMSTK Record models."""

    def set_attributes(self, attributes):
        """Set one or more RAMSTK<Table> attributes.

        .. note:: you should pop the primary and foreign key entries from the
            attributes dict before passing it to this method.

        :param attributes: dict of key:value pairs to assign to the
            instance attributes.
        :return: None
        :rtype: None
        :raise: AttributeError if passed an attribute key that doesn't exist as
            a table field.
        """
        for _key in attributes:
            getattr(self, _key)
            setattr(
                self, _key, none_to_default(attributes[_key], self.__defaults__[_key])
            )


class RAMSTKBaseTable:
    """Meta-class for all RAMSTK Table Models.

    :ivar tree: the treelib Tree()that will contain the structure of the RAMSTK
        module being modeled.
    :type tree: :class:`treelib.Tree`
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _db_id_colname: str = ""
    _db_tablename: str = ""
    _root: int = 0
    _select_msg: str = "selected_revision"
    _tag: str = ""

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    # pylint: disable=unused-argument
    # noinspection PyUnusedLocal
    def __init__(self, **kwargs: Dict[str, Any]) -> None:
        """Initialize an RAMSTK data model instance."""
        # Initialize private dictionary attributes.
        self._fkey: Dict[str, int] = {}
        self._pkey: Dict[str, List[str]] = {}

        # Initialize private list attributes.
        self._lst_id_columns: List[str] = []

        # Initialize private scalar attributes.
        self._parent_id: int = 0
        self._record: Type[object]
        self._revision_id: int = 0

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.dao: BaseDatabase = BaseDatabase()
        self.last_id: int = 0
        self.pkey: str = ""
        self.tree: treelib.Tree = treelib.Tree()
        self.do_get_new_record: Callable[[Dict[str, Any]], object]

        # Add the root to the Tree().  This is necessary to allow multiple
        # entries at the top level as there can only be one root in a treelib
        # Tree().  Manipulation and viewing of a RAMSTK module tree needs to
        # ignore the root of the tree.
        self.tree.create_node(tag=self._tag, identifier=self._root)

        # Subscribe to PyPubSub messages.
        pub.subscribe(self.do_connect, "succeed_connect_program_database")
        pub.subscribe(self.do_delete, f"request_delete_{self._tag}")
        pub.subscribe(self.do_get_attributes, f"request_get_{self._tag}_attributes")
        pub.subscribe(self.do_get_tree, f"request_get_{self._tag}_tree")
        pub.subscribe(self.do_insert, f"request_insert_{self._tag}")
        pub.subscribe(self.do_select_all, self._select_msg)
        pub.subscribe(self.do_set_attributes, f"request_set_{self._tag}_attributes")
        pub.subscribe(self.do_set_attributes, f"lvw_editing_{self._tag}")
        pub.subscribe(self.do_set_attributes, f"mvw_editing_{self._tag}")
        pub.subscribe(self.do_set_attributes, f"wvw_editing_{self._tag}")
        pub.subscribe(
            self.do_set_attributes_all,
            f"request_set_all_{self._tag}_attributes",
        )
        pub.subscribe(self.do_set_tree, f"succeed_calculate_{self._tag}")
        pub.subscribe(self.do_update, f"request_update_{self._tag}")
        pub.subscribe(self.do_update_all, f"request_update_all_{self._tag}s")
        pub.subscribe(self.do_update_all, "request_save_project")

    def do_connect(self, dao: BaseDatabase) -> None:
        """Connect data manager to a database.

        :param dao: the BaseDatabase() instance (data access object)
            representing the connected RAMSTK Program database.
        :type dao: :class:`ramstk.db.base.BaseDatabase`
        """
        self.dao = dao

    def do_create_all_codes(self, prefix: str) -> None:
        """Create codes for all MODULE data table records.

        :param prefix: the string to use as a prefix for each code.
        :return: None
        :rtype: None
        """
        for _node in self.tree.all_nodes():
            # noinspection PyUnresolvedReferences
            self.do_create_code(_node.identifier, prefix)  # type: ignore

    def do_delete(self, node_id: int) -> None:
        """Remove a record from the Program database and records tree.

        :param node_id: the ID of the record to delete.
        :return: None
        :rtype: None
        """
        try:
            for _node in self.tree.children(node_id):
                _record = self.do_select(_node.identifier)
                self.dao.do_delete(_record)

            _record = self.do_select(node_id)
            self.dao.do_delete(_record)

            self.tree.remove_node(node_id)
            self.last_id = self.dao.get_last_id(self._db_tablename, self._db_id_colname)

            pub.sendMessage(
                f"succeed_delete_{self._tag}",
                tree=self.tree,
            )
        except (AttributeError, DataAccessError, NodeIDAbsentError):
            _error_msg: str = (
                f"Attempted to delete non-existent "
                f"{self._tag.replace('_', ' ').title()} ID {node_id}."
            )
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=_error_msg,
            )
            pub.sendMessage(
                f"fail_delete_{self._tag}",
                error_message=_error_msg,
            )

    def do_get_attributes(self, node_id: int, table: str = "") -> None:
        """Retrieve the RAMSTK data table attributes for node ID.

        :param node_id: the node ID in the treelib Tree to get the
            attributes for.
        :param table: the RAMSTK data table to retrieve the attributes from.
        :return: None
        :rtype: None
        """
        try:
            pub.sendMessage(
                f"succeed_get_{self._tag}_attributes",
                attributes=self.do_select(node_id).get_attributes(),
            )
        except AttributeError:
            _method_name = inspect.currentframe().f_code.co_name  # type: ignore
            _error_msg = (
                f"{_method_name}: No attributes found for record ID {node_id} in "
                f"{self._tag} table."
            )
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=_error_msg,
            )
            pub.sendMessage(
                f"fail_get_{table}_attributes",
                error_message=_error_msg,
            )

    def do_get_tree(self) -> None:
        """Retrieve the records tree.

        :return: None
        :rtype: None
        """
        pub.sendMessage(
            f"succeed_get_{self._tag}_tree",
            tree=self.tree,
        )

    def do_insert(self, attributes: Dict[str, Any]) -> None:
        """Add a new record to the RAMSTK program database and records tree.

        :param attributes: the attribute values to assign to the new record.
        :return: None
        :rtype: None
        """
        _method_name = inspect.currentframe().f_code.co_name  # type: ignore

        try:
            _record = self.do_get_new_record(attributes)
            for _id in self._lst_id_columns:
                attributes.pop(_id)

            _record.set_attributes(attributes)  # type: ignore
            _identifier = self.last_id + 1

            self.dao.do_insert(_record)
            self.last_id = self.dao.get_last_id(self._db_tablename, self._db_id_colname)

            self.tree.create_node(
                tag=self._tag,
                identifier=_identifier,
                parent=self._parent_id,
                data={self._tag: _record},
            )

            pub.sendMessage(
                f"succeed_insert_{self._tag}",
                tree=self.tree,
            )
        except DataAccessError as _error:
            _error_msg = f"{_method_name}: {_error.msg}"
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=_error_msg,
            )
            pub.sendMessage(
                f"fail_insert_{self._tag}",
                error_message=_error.msg,
            )
        except NodeIDAbsentError as _error:
            _error_msg = f"{_method_name}: {_error}"
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=str(_error),
            )
            pub.sendMessage(
                f"fail_insert_{self._tag}",
                error_message=_error_msg,
            )

    def do_select(self, node_id: Any) -> Any:
        """Retrieve the RAMSTK data table record for the Node ID passed.

        :param node_id: the Node ID of the data package to retrieve.
        :return: the instance of the RAMSTK<MODULE> data table that was
            requested or None if the requested Node ID does not exist.
        :raise: KeyError if passed the name of a table that isn't managed by
            this manager.
        """
        try:
            _entity = self.tree.get_node(node_id).data[self._tag]
        except (AttributeError, KeyError, treelib.tree.NodeIDAbsentError, TypeError):
            _method_name = inspect.currentframe().f_code.co_name  # type: ignore
            _error_msg: str = (
                f"{_method_name}: No data package for node ID {node_id} in module "
                f"{self._tag}."
            )
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=_error_msg,
            )
            _entity = None

        return _entity

    def do_select_all(self, attributes: Dict[str, Any]) -> None:
        """Retrieve all the records from the RAMSTK Program database.

        :param attributes: the attributes dict for the selected failure action.
        :return: None
        :rtype: None
        """
        self.tree = do_clear_tree(self.tree)

        try:
            self._revision_id = attributes["revision_id"]
        except KeyError:
            try:
                self._revision_id = attributes["site_id"]
            except KeyError:
                self._revision_id = 0

        for _record in self.dao.do_select_all(
            self._record,
            key=[
                self._lst_id_columns[0],
            ],
            value=[
                self._revision_id,
            ],
            order=self._db_id_colname,
        ):
            try:
                self._parent_id = _record.get_attributes()["parent_id"]
            except KeyError:
                self._parent_id = 0

            self.tree.create_node(
                tag=self._tag,
                identifier=_record.get_attributes()[self.pkey],
                parent=self._parent_id,
                data={self._tag: _record},
            )
        self.last_id = self.dao.get_last_id(self._db_tablename, self._db_id_colname)

        pub.sendMessage(
            f"succeed_retrieve_{self._tag}s",
            tree=self.tree,
        )

    def do_set_attributes(self, node_id: List, package: Dict[str, Any]) -> None:
        """Set the attributes of the record associated with node ID.

        :param node_id: the ID of the record in the RAMSTK Program database
            table whose attributes are to be set.
        :param package: the key:value pair of the attribute to set.
        :return: None
        :rtype: None
        """
        [[_key, _value]] = package.items()
        # ISSUE: Make node_id an integer argument to do_set_attributes()
        #
        # The node_id argument to RAMSTKBaseTable.do_set_attributes() is currently a
        # List type argument.  This is deprecated and all calls should replace this
        # with an integer argument.  After fixing all calls, remove the try construct
        # in do_set_attributes() that exists to handle the List and int types.
        # labels: type: refactor
        try:
            _node_id = node_id[0]
        except TypeError:
            _node_id = node_id

        try:
            _attributes = self.do_select(_node_id).get_attributes()
        except (AttributeError, KeyError):
            _method_name = inspect.currentframe().f_code.co_name  # type: ignore
            _error_msg: str = (
                f"{_method_name}: No data package for node ID {_node_id} in module "
                f"{self._tag}."
            )
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=_error_msg,
            )
            _attributes = {}

        for _field in self._lst_id_columns:
            try:
                _attributes.pop(_field)
            except KeyError:
                pass

        if _key in _attributes:
            _attributes[_key] = _value

            self.do_select(_node_id).set_attributes(_attributes)

        # noinspection PyUnresolvedReferences
        self.do_get_tree()  # type: ignore

    def do_set_attributes_all(self, attributes: Dict[str, Any]) -> None:
        """Set all the attributes of the record associated with the Module ID.

        :param attributes: the aggregate attributes dict for the allocation
            item.
        :return: None
        :rtype: None
        """
        for _key in attributes:
            self.do_set_attributes(
                node_id=[attributes[self.pkey]],
                package={_key: attributes[_key]},
            )

    def do_set_tree(self, tree: treelib.Tree) -> None:
        """Set the MODULE treelib Tree().

        This method is generally used to respond to events such as successful
        calculations of the entire system.

        :param tree: the treelib Tree() to assign to the tree attribute.
        :return: None
        :rtype: None
        """
        self.tree = tree

    def do_update(self, node_id: int) -> None:
        """Update record associated with node ID in RAMSTK Program database.

        :param node_id: the node ID of the record to save.
        :return: None
        :rtype: None
        """
        _fail_topic = f"fail_update_{self._tag}"
        _method_name: str = inspect.currentframe().f_code.co_name  # type: ignore

        try:
            self.dao.do_update(self.tree.get_node(node_id).data[self._tag])
            pub.sendMessage(
                f"succeed_update_{self._tag}",
                tree=self.tree,
            )
        except AttributeError:
            _error_msg: str = (
                f"{_method_name}: Attempted to save non-existent "
                f"{self._tag.replace('_', ' ')} with {self._tag.replace('_', ' ')} "
                f"ID {node_id}."
            )
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=_error_msg,
            )
            pub.sendMessage(
                _fail_topic,
                error_message=_error_msg,
            )
        except KeyError:
            _error_msg = (
                f"{_method_name}: No data package found for "
                f"{self._tag.replace('_', ' ')} ID {node_id}."
            )
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=_error_msg,
            )
            pub.sendMessage(
                _fail_topic,
                error_message=_error_msg,
            )
        except (DataAccessError, TypeError):
            if node_id != 0:
                _error_msg = (
                    f"{_method_name}: The value for one or more attributes for "
                    f"{self._tag.replace('_', ' ')} ID {node_id} was the wrong type."
                )
            else:
                _error_msg = (
                    f"{_method_name}: Attempting to update the root node {node_id}."
                )
            pub.sendMessage(
                "do_log_debug",
                logger_name="DEBUG",
                message=_error_msg,
            )
            pub.sendMessage(
                _fail_topic,
                error_message=_error_msg,
            )

        pub.sendMessage("request_set_cursor_active")

    # noinspection PyUnresolvedReferences
    # pylint: disable=no-value-for-parameter
    def do_update_all(self) -> None:
        """Update all MODULE data table records in the RAMSTK Program database.

        :return: None
        :rtype: None
        """
        for _node in self.tree.all_nodes():
            self.do_update(_node.identifier)  # type: ignore

        pub.sendMessage("request_set_cursor_active")
        pub.sendMessage("succeed_update_all")


class RAMSTKBaseView:
    """Meta-class for all RAMSTK View models."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _root = 0
    _tag = ""

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    # pylint: disable=unused-argument
    # noinspection PyUnusedLocal
    def __init__(self, **kwargs: Dict[str, Any]) -> None:
        """Initialize a RAMSTK view model instance."""
        # Initialize private dictionary attributes.
        self._dic_load_functions: Dict[str, Callable[..., object]] = {}
        self._dic_trees: Dict[str, treelib.Tree] = {}

        # Initialize private list attributes.
        self._lst_modules: List[str] = []

        # Initialize private scalar attributes.
        self._revision_id: int = 0

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.dao: BaseDatabase = BaseDatabase()
        self.tree: treelib.Tree = treelib.Tree()

        # Add the root to the Tree().  This is necessary to allow multiple
        # entries at the top level as there can only be one root in a treelib
        # Tree().  Manipulation and viewing of a RAMSTK module tree needs to
        # ignore the root of the tree.
        self.tree.create_node(tag=self._tag, identifier=self._root)

        # Subscribe to PyPubSub messages.

    def do_set_tree(self, tree: treelib.Tree) -> None:
        """Assign the treelib Tree() for the constituent module.

        :param tree: the calling module's treelib Tree().
        :return: None
        :rtype: None
        """
        self._dic_trees[tree.get_node(0).tag] = tree
        self.on_select_all()

    def on_select_all(self) -> None:
        """Build the usage profile treelib Tree().

        This method builds the hierarchical treelib Tree() from the individual
        module trees.

        :return: None
        :rtype: None
        """
        for _node in self.tree.children(self.tree.root):
            self.tree.remove_node(_node.identifier)

        if self._dic_trees[self._lst_modules[0]].depth() > 0:
            self._dic_load_functions[self._lst_modules[0]]()  # type: ignore

            pub.sendMessage(
                f"succeed_retrieve_{self._tag}",
                tree=self.tree,
            )



# Standard Library Imports
from typing import Any, Optional, Tuple

# RAMSTK Package Imports
from ramstk.db.base import BaseDatabase as BaseDatabase
from ramstk.views.gtk3 import GdkPixbuf as GdkPixbuf
from ramstk.views.gtk3 import GObject as GObject
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import Pango as Pango
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets.button import RAMSTKButton as RAMSTKButton
from ramstk.views.gtk3.widgets.button import RAMSTKCheckButton as RAMSTKCheckButton
from ramstk.views.gtk3.widgets.combo import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets.entry import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets.label import do_make_label_group as do_make_label_group

class RAMSTKDialog(Gtk.Dialog):
    def __init__(self, dlgtitle: str, **kwargs: Any) -> None: ...
    def do_destroy(self) -> None: ...
    def do_run(self) -> Any: ...

class RAMSTKDatabaseSelect(RAMSTKDialog):
    _dao: Any = ...
    _dic_icons: Any = ...
    _lst_databases: Any = ...
    _lst_labels: Any = ...
    _old_host: str = ...
    btnRefresh: Any = ...
    btnSave: Any = ...
    cmbDialect: Any = ...
    txtHost: Any = ...
    txtPort: Any = ...
    txtDatabase: Any = ...
    txtUser: Any = ...
    txtPassword: Any = ...
    tvwTreeView: Any = ...
    _lst_widgets: Any = ...
    database: Any = ...
    exists: bool = ...
    def __init__(self, dlgtitle: str, **kwargs: Any) -> None: ...
    def do_run(self) -> Gtk.ResponseType: ...
    def _get_database(self) -> None: ...
    def _request_load_databases(self, __button: RAMSTKButton) -> None: ...
    def __do_load_combobox(self) -> None: ...
    def __do_load_databases(self) -> None: ...
    def __do_set_callbacks(self) -> None: ...
    def __do_set_properties(self) -> None: ...
    def __make_ui(self) -> None: ...
    def __on_row_change(self, selection: Gtk.TreeSelection) -> None: ...

class RAMSTKDateSelect(Gtk.Dialog):
    _calendar: Any = ...
    def __init__(self, **kwargs: Any) -> None: ...
    def do_destroy(self) -> None: ...
    def do_run(self) -> Any: ...

class RAMSTKFileChooser(Gtk.FileChooserDialog):
    def __init__(self, title: str, parent: object) -> None: ...
    def do_destroy(self) -> None: ...
    def do_run(self) -> Tuple[Optional[Any], Optional[Any]]: ...

class RAMSTKMessageDialog(Gtk.MessageDialog):
    def __init__(self, parent: Gtk.Window = ...) -> None: ...
    def do_set_message(self, message: str) -> None: ...
    def do_set_message_type(self, message_type: str = ...) -> None: ...
    def do_run(self) -> Any: ...
    def do_destroy(self) -> None: ...



# pylint: disable=unused-import, wrong-import-position
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.usage_profile.__init__.py is part of The RAMSTK
#       Project
#
# All rights reserved.
# Copyright 2007 - 2020 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK GTK3 usage profile package."""

# RAMSTK Local Imports
from .panel import UsageProfileTreePanel
from .view import UsageProfileListView



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hazard_analysis.panel.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Hazard Analysis Panels."""

# Standard Library Imports
from typing import Any, Dict, List, Tuple

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import RAMSTKTreePanel


class HazardsTreePanel(RAMSTKTreePanel):
    """The panel to display the hazards analysis for the selected Function."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _select_msg = "succeed_retrieve_hazards"
    _tag = "hazard"
    _title: str = _("Hazards Analysis")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Function Hazard Analysis panel."""
        super().__init__()

        # Initialize private dict instance attributes.
        self.tvwTreeView.dic_row_loader = {
            "hazard": super().do_load_treerow,
        }

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.
        self._filtered_tree = True
        self._on_edit_message: str = f"wvw_editing_{self._tag}"

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map = {
            "revision_id": [
                0,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Revision ID"),
                "gint",
            ],
            "function_id": [
                1,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Function ID"),
                "gint",
            ],
            "hazard_id": [
                2,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Hazard ID"),
                "gint",
            ],
            "potential_hazard": [
                3,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Potential Hazard"),
                "gchararray",
            ],
            "potential_cause": [
                4,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Potential Cause"),
                "gchararray",
            ],
            "assembly_effect": [
                5,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Assembly Effect"),
                "gchararray",
            ],
            "assembly_severity": [
                6,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Assembly Severity"),
                "gchararray",
            ],
            "assembly_probability": [
                7,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Assembly Probability"),
                "gchararray",
            ],
            "assembly_hri": [
                8,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Assembly HRI"),
                "gint",
            ],
            "assembly_mitigation": [
                9,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Assembly Mitigation"),
                "gchararray",
            ],
            "assembly_severity_f": [
                10,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Final Assembly Severity"),
                "gchararray",
            ],
            "assembly_probability_f": [
                11,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Final Assembly Probability"),
                "gchararray",
            ],
            "assembly_hri_f": [
                12,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Final Assembly HRI"),
                "gint",
            ],
            "system_effect": [
                13,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("System Effect"),
                "gchararray",
            ],
            "system_severity": [
                14,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("System Severity"),
                "gchararray",
            ],
            "system_probability": [
                15,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("System Probability"),
                "gchararray",
            ],
            "system_hri": [
                16,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("System HRI"),
                "gint",
            ],
            "system_mitigation": [
                17,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("System Mitigation"),
                "gchararray",
            ],
            "system_severity_f": [
                18,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Final System Severity"),
                "gchararray",
            ],
            "system_probability_f": [
                19,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Final System Probability"),
                "gchararray",
            ],
            "system_hri_f": [
                20,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Final System HRI"),
                "gint",
            ],
            "remarks": [
                21,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Remarks"),
                "gchararray",
            ],
            "function_1": [
                22,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Function 1"),
                "gchararray",
            ],
            "function_2": [
                23,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Function 2"),
                "gchararray",
            ],
            "function_3": [
                24,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Function 3"),
                "gchararray",
            ],
            "function_4": [
                25,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Function 4"),
                "gchararray",
            ],
            "function_5": [
                26,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Function 5"),
                "gchararray",
            ],
            "result_1": [
                27,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Result 1"),
                "gfloat",
            ],
            "result_2": [
                28,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Result 2"),
                "gfloat",
            ],
            "result_3": [
                29,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Result 3"),
                "gfloat",
            ],
            "result_4": [
                30,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Result 4"),
                "gfloat",
            ],
            "result_5": [
                31,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Result 5"),
                "gfloat",
            ],
            "user_blob_1": [
                32,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Text 1"),
                "gchararray",
            ],
            "user_blob_2": [
                33,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Text 2"),
                "gchararray",
            ],
            "user_blob_3": [
                34,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Text 3"),
                "gchararray",
            ],
            "user_float_1": [
                35,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Float 1"),
                "gfloat",
            ],
            "user_float_2": [
                36,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Float 2"),
                "gfloat",
            ],
            "user_float_3": [
                37,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Float 3"),
                "gfloat",
            ],
            "user_int_1": [
                38,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Integer 1"),
                "gint",
            ],
            "user_int_2": [
                39,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Integer 2"),
                "gint",
            ],
            "user_int_3": [
                40,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("User Integer 3"),
                "gint",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.tvwTreeView.set_tooltip_text(
            _("Displays the Hazards Analysis for the currently selected Function.")
        )

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_load_panel, f"succeed_calculate_{self._tag}")

        pub.subscribe(self._on_select_function, "selected_function")

    # pylint: disable=unused-argument
    # noinspection PyUnusedLocal
    def filter_tree(self, model: Gtk.TreeModel, row: Gtk.TreeIter, data: Any) -> bool:
        """Filter Hazards to show only those associated with the selected Function.

        :param model: the filtered model for the Hazard RAMSTKTreeView.
        :param row: the iter to check against condition(s).
        :param data: unused in this method; required by Gtk.TreeModelFilter() widget.
        :return: True if row should be visible, False else.
        :rtype: bool
        """
        return model[row][1] == self._parent_id

    def do_load_severity(self, criticalities: Dict[int, Tuple[str, str, int]]) -> None:
        """Load the Gtk.CellRendererCombo() containing severities.

        :param criticalities: the dict containing the hazard severity
            categories and values.
        :return: None
        :rtype: None
        """
        # ISSUE: Replace Hazards Panel Integer Position List with String Key List
        #
        # The list of column positions should be replaced with a list of column keys
        # and the position should be taken from self.tvwTreeView.position[_key].
        # This needs to be done for all the methods that load a Gtk.CellRendererCombo().
        # labels: type: refactor
        for i in [6, 10, 14, 18]:
            _model = self.tvwTreeView.get_cell_model(i)
            for _key in criticalities:
                _model.append((criticalities[_key][1],))

    def do_load_hazards(self, hazards: Dict[Any, Any]) -> None:
        """Load the Gtk.CellRendererCombos() containing hazards.

        :param hazards: the dict containing the hazards and hazard types
            to be considered.
        :return: None
        :rtype: None
        """
        _model = self.tvwTreeView.get_cell_model(3)
        for _key in hazards:
            _hazard = f"{hazards[_key][0]}, {hazards[_key][1]}"
            _model.append((_hazard,))

    def do_load_probability(self, probabilities: List[str]) -> None:
        """Load the Gtk.CellRendererCombos() containing probabilities.

        :param probabilities: the list of hazard probabilities.
        :return: None
        :rtype: None
        """
        for i in [7, 11, 15, 19]:
            _model = self.tvwTreeView.get_cell_model(i)
            for _probability in probabilities:
                _model.append((_probability[0],))

    def _on_row_change(self, selection: Gtk.TreeSelection) -> None:
        """Handle events for the HazOps Tree View RAMSTKTreeView().

        This method is called whenever a Tree View row is activated.

        :param selection: the HazOps RAMSTKTreeview Gtk.TreeSelection().
        :return: None
        :rtype: None
        """
        _attributes = super().on_row_change(selection)

        if _attributes:
            self._record_id = _attributes["hazard_id"]

            pub.sendMessage(
                "selected_hazard",
                attributes=_attributes,
            )

    def _on_select_function(self, attributes: Dict[str, Any]) -> None:
        """Filter hazards list when Function is selected.

        :param attributes: the dict of Function attributes for the selected Function.
        :return: None
        :rtype: None
        """
        self._parent_id = attributes["function_id"]
        self.tvwTreeView.filt_model.refilter()



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hardware.components.integrated_circuit.py is part of the
#       RAMSTK Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Integrated Circuit Results Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.milhdbk217f import MilHdbk217FResultPanel
from ramstk.views.gtk3.widgets import RAMSTKEntry


class ICMilHdbk217FResultPanel(MilHdbk217FResultPanel):
    """Display IC assessment results attribute data in the RAMSTK Work Book.

    The Integrated Circuit assessment result view displays all the assessment
    results for the selected integrated circuit.  This includes, currently,
    results for MIL-HDBK-217FN2 parts count and MIL-HDBK-217FN2 part stress
    methods.  The attributes of a integrated circuit assessment result view
    are:

    :ivar txtC1: displays the die complexity hazard rate of the integrated
        circuit.
    :ivar txtPiT: displays the temperature factor for the integrated circuit.
    :ivar txtC2: displays the package failure rate for the integrated circuit.
    :ivar txtPiL: displays the learning factor for the integrated circuit.
    :ivar txtLambdaCYC: displays the read/write cycling induced hazard rate for
        the EEPROM.
    :ivar txtLambdaBD: displays the die base hazard rate for the VLSI device.
    :ivar txtPiMFG: displays the manufacturing process correction factor for
        VLSI device.
    :ivar txtPiCD: displays the die complexity correction factor for the VLSI
        device.
    :ivar txtLambdaBP: displays the package base hazard rate for the VLSI
        device.
    :ivar txtPiPT: displays the package type correction factor for the VLSI
        device.
    :ivar txtLambdaEOS: displays the electrical overstress hazard rate for the
        VLSI device.
    :ivar txtPiA: displays the application factor for the integrated circuit.
    """

    _lambda_p = '<span foreground="blue">\u03BB<sub>p</sub> = '
    _function_1 = "(C<sub>1</sub>\u03C0<sub>T</sub> + C<sub>2</sub>\u03C0<sub>E</sub>)"
    _function_2 = "\u03C0<sub>Q</sub>\u03C0<sub>L</sub></span>"
    _function_3 = "\u03BB<sub>CYC</sub>)" + _function_2

    # Define private class dict class attributes.
    _dic_part_stress: Dict[int, str] = {
        1: _lambda_p + _function_1 + _function_2,
        2: _lambda_p + _function_1 + _function_2,
        3: _lambda_p + _function_1 + _function_2,
        4: _lambda_p + _function_1 + _function_2,
        5: _lambda_p + _function_1 + _function_3,
        6: _lambda_p + _function_1 + _function_3,
        7: _lambda_p + _function_1 + _function_3,
        8: _lambda_p + _function_1 + _function_3,
        9: _lambda_p + "(C<sub>1</sub>\u03C0<sub>T</sub>\u03C0<sub>A</sub> + "
        "C<sub>2</sub>\u03C0<sub>E</sub>)\u03C0<sub>L</sub>\u03C0<sub>Q</sub"
        "></span>",
        10: _lambda_p
        + "\u03BB<sub>BD</sub>\u03C0<sub>MFG</sub>\u03C0<sub>T</sub>\u03C0<sub"
        ">CD</sub> + \u03BB<sub>BP</sub>\u03C0<sub>E</sub>\u03C0<sub>Q</sub"
        ">\u03C0<sub>PT</sub> + \u03BB<sub>EOS</sub></span> ",
    }

    # Define private class list class attributes.

    # Define private scalar class attributes.
    _title: str = _("Integrated Circuit MIL-HDBK-217F Results")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the IC assessment result view."""
        super().__init__()

        # Initialize widgets.
        self.txtC1: RAMSTKEntry = RAMSTKEntry()
        self.txtC2: RAMSTKEntry = RAMSTKEntry()
        self.txtLambdaBD: RAMSTKEntry = RAMSTKEntry()
        self.txtLambdaBP: RAMSTKEntry = RAMSTKEntry()
        self.txtLambdaCYC: RAMSTKEntry = RAMSTKEntry()
        self.txtLambdaEOS: RAMSTKEntry = RAMSTKEntry()
        self.txtPiA: RAMSTKEntry = RAMSTKEntry()
        self.txtPiCD: RAMSTKEntry = RAMSTKEntry()
        self.txtPiL: RAMSTKEntry = RAMSTKEntry()
        self.txtPiMFG: RAMSTKEntry = RAMSTKEntry()
        self.txtPiPT: RAMSTKEntry = RAMSTKEntry()
        self.txtPiT: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "hazard_rate_model": [
                13,
                self.lblModel,
                "",
                None,
                "",
                "",
                {
                    "tooltip": _(
                        "The assessment model used to calculate the integrated circuit "
                        "hazard rate."
                    ),
                },
                "",
            ],
            "lambda_b": [
                23,
                self.txtLambdaB,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _("The base hazard rate for the integrated circuit."),
                },
                "\u03BB<sub>b</sub>:",
            ],
            "pi_q": [
                30,
                self.txtPiQ,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The quality factor for the integrated circuit."),
                },
                "\u03C0<sub>Q</sub>:",
            ],
            "pi_e": [
                19,
                self.txtPiE,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The environment factor for the integrated circuit."),
                },
                "\u03C0<sub>E</sub>:",
            ],
            "C1": [
                6,
                self.txtC1,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The die complexity factor for the integrated circuit."
                    ),
                },
                "C1:",
            ],
            "pi_t": [
                33,
                self.txtPiT,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The temperature correction factor for the integrated circuit."
                    ),
                },
                "\u03C0<sub>T</sub>:",
            ],
            "C2": [
                7,
                self.txtC2,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The package hazard rate for the integrated circuit."),
                },
                "C2:",
            ],
            "pi_l": [
                23,
                self.txtPiL,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The learning factor for the integrated circuit."),
                },
                "\u03C0<sub>L</sub>:",
            ],
            "lambdaCYC": [
                10,
                self.txtLambdaCYC,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The read/write cycling induced hazard rate for the integrated "
                        "circuit."
                    ),
                },
                "\u03BB<sub>CYC</sub>:",
            ],
            "lambdaBD": [
                8,
                self.txtLambdaBD,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The die base hazard rate for the integrated circuit."
                    ),
                },
                "\u03BB<sub>BD</sub>:",
            ],
            "pi_mfg": [
                25,
                self.txtPiMFG,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The manufacturing process correction factor for the "
                        "integrated circuit."
                    ),
                },
                "\u03C0<sub>MFG</sub>:",
            ],
            "pi_cd": [
                14,
                self.txtPiCD,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The die complexity correction factor for the integrated "
                        "circuit."
                    ),
                },
                "\u03C0<sub>CD</sub>:",
            ],
            "lambdaBP": [
                9,
                self.txtLambdaBP,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The package base hazard rate for the integrated circuit."
                    ),
                },
                "\u03BB<sub>BP</sub>:",
            ],
            "pi_pt": [
                29,
                self.txtPiPT,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The package type factor for the integrated circuit."),
                },
                "\u03C0<sub>PT</sub>:",
            ],
            "lambdaEOS": [
                11,
                self.txtLambdaEOS,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The electrical overstress hazard rate for the integrated "
                        "circuit."
                    ),
                },
                "\u03BB<sub>EOS</sub>:",
            ],
            "pi_a": [
                12,
                self.txtPiA,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The application factor for the integrated circuit."),
                },
                "\u03C0<sub>A</sub>:",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        super().do_set_properties()
        super().do_make_panel()

        # Subscribe to PyPubSub messages.

    def _do_load_entries(self, attributes: Dict[str, Any]) -> None:
        """Load the integrated circuit assessment results page.

        :param attributes: the attributes dictionary for the selected
            Integrated Circuit.
        :return: None
        :rtype: None
        """
        super().do_load_entries(attributes)

        self.txtC1.set_sensitive(False)
        self.txtPiT.set_sensitive(False)
        self.txtC2.set_sensitive(False)
        self.txtPiL.set_sensitive(False)
        self.txtLambdaCYC.set_sensitive(False)
        self.txtLambdaBD.set_sensitive(False)
        self.txtPiMFG.set_sensitive(False)
        self.txtPiCD.set_sensitive(False)
        self.txtLambdaBP.set_sensitive(False)
        self.txtPiPT.set_sensitive(False)
        self.txtLambdaEOS.set_sensitive(False)
        self.txtPiA.set_sensitive(False)

        if self.category_id == 1 and self._hazard_rate_method_id == 2:
            self.txtC1.do_update(str(self.fmt.format(attributes["C1"])))
            self.txtPiT.do_update(str(self.fmt.format(attributes["piT"])))
            self.txtC2.do_update(str(self.fmt.format(attributes["C2"])))
            self.txtPiL.do_update(str(self.fmt.format(attributes["piL"])))
            self.txtLambdaCYC.do_update(
                str(self.fmt.format(attributes["lambdaCYC"])),
            )
            self.txtLambdaBD.do_update(str(self.fmt.format(attributes["lambdaBD"])))
            self.txtPiMFG.do_update(str(self.fmt.format(attributes["piMFG"])))
            self.txtPiCD.do_update(str(self.fmt.format(attributes["piCD"])))
            self.txtLambdaBP.do_update(str(self.fmt.format(attributes["lambdaBP"])))
            self.txtPiPT.do_update(str(self.fmt.format(attributes["piPT"])))
            self.txtLambdaEOS.do_update(
                str(self.fmt.format(attributes["lambdaEOS"])),
            )
            self.txtPiA.do_update(str(self.fmt.format(attributes["piA"])))



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.allocation.view.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Allocation Views."""

# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import RAMSTKPanel, RAMSTKWorkView

# RAMSTK Local Imports
from . import AllocationGoalMethodPanel, AllocationTreePanel


class AllocationWorkView(RAMSTKWorkView):
    """Display Allocation attribute data in the RAMSTK Work Book.

    The Allocation Work View displays all the allocation data attributes for
    the selected hardware item. The attributes of an Allocation General Data
    Work View are:

    :cvar _tag: the name of the module.

    :ivar _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "allocation"
    _tablabel: str = _("Allocation")
    _tabtooltip: str = _(
        "Displays the Allocation analysis for the selected hardware item."
    )

    # Define public dictionary class attributes.

    # Define public dictionary list attributes.

    # Define public dictionary scalar attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Allocation Work View general data page.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_callbacks.insert(0, self._do_request_calculate)
        self._lst_icons.insert(0, "calculate")
        self._lst_mnu_labels.insert(0, _("Calculate"))
        self._lst_tooltips = [
            _("Calculate the currently selected Allocation line item."),
            _("Save changes to the currently selected Allocation line item."),
            _("Save changes to all Allocation line items."),
        ]

        # Initialize private scalar attributes.
        self._pnlGoalMethods: RAMSTKPanel = AllocationGoalMethodPanel()
        self._pnlPanel: RAMSTKPanel = AllocationTreePanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, "selected_hardware")

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the allocation's record ID.

        :param attributes: the attributes dict for the selected allocation
            record.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["parent_id"] = attributes["parent_id"]
        self.dic_pkeys["record_id"] = attributes["hardware_id"]

    def _do_request_calculate(self, __button: Gtk.ToolButton) -> None:
        """Calculate the Allocation reliability metrics.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        :rtype: None
        """
        super().do_set_cursor_busy()
        pub.sendMessage(
            "request_calculate_allocation", node_id=self.dic_pkeys["record_id"]
        )

    def __make_ui(self) -> None:
        """Build the user interface for the allocation tab.

        :return: None
        :rtype: None
        """
        _hpaned: Gtk.HPaned = super().do_make_layout_lr()

        self._pnlGoalMethods.fmt = self.fmt
        self._pnlGoalMethods.do_load_comboboxes()

        super().do_embed_treeview_panel()

        self.remove(self.get_children()[-1])
        _hpaned.pack1(self._pnlGoalMethods, True, True)
        _hpaned.pack2(self._pnlPanel, True, True)

        self.show_all()



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.revision.panel.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Revision Panels."""

# Third Party Imports
from pubsub import pub  # type: ignore

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import (
    RAMSTKEntry,
    RAMSTKFixedPanel,
    RAMSTKTextView,
    RAMSTKTreePanel,
)


class RevisionTreePanel(RAMSTKTreePanel):
    """Panel to display flat list of revisions."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _select_msg = "succeed_retrieve_revisions"
    _tag = "revision"
    _title = _("List of Revisions")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Revision panel."""
        super().__init__()

        # Initialize private dictionary class attributes.
        self.tvwTreeView.dic_row_loader = {
            "revision": super().do_load_treerow,
        }

        # Initialize private list class attributes.

        # Initialize private scalar class attributes.
        self._on_edit_message: str = f"mvw_editing_{self._tag}"

        # Initialize public dictionary class attributes.
        self.dic_attribute_widget_map = {
            "revision_id": [
                0,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Revision ID"),
                "gint",
            ],
            "availability_logistics": [
                1,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Logistics A(t)"),
                "gfloat",
            ],
            "availability_mission": [
                2,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Mission A(t)"),
                "gfloat",
            ],
            "cost": [
                3,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Cost"),
                "gfloat",
            ],
            "cost_failure": [
                4,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Cost/Failure"),
                "gfloat",
            ],
            "cost_hour": [
                5,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Cost/Hour"),
                "gfloat",
            ],
            "hazard_rate_active": [
                6,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Active h(t)"),
                "gfloat",
            ],
            "hazard_rate_dormant": [
                7,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Dormant h(t)"),
                "gfloat",
            ],
            "hazard_rate_logistics": [
                8,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Logistics h(t)"),
                "gfloat",
            ],
            "hazard_rate_mission": [
                9,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Mission h(t)"),
                "gfloat",
            ],
            "hazard_rate_software": [
                10,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Software h(t)"),
                "gfloat",
            ],
            "mmt": [
                11,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("MMT"),
                "gfloat",
            ],
            "mcmt": [
                12,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("MCMT"),
                "gfloat",
            ],
            "mpmt": [
                13,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("MPMT"),
                "gfloat",
            ],
            "mtbf_logistics": [
                14,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Logistics MTBF"),
                "gfloat",
            ],
            "mtbf_mission": [
                15,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Mission MTBF"),
                "gfloat",
            ],
            "mttr": [
                16,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("MTTR"),
                "gfloat",
            ],
            "name": [
                17,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Revision Name"),
                "gchararray",
            ],
            "reliability_logistics": [
                18,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Logistics R(t)"),
                "gfloat",
            ],
            "reliability_mission": [
                19,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                1.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Mission R(t)"),
                "gfloat",
            ],
            "remarks": [
                20,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Remarks"),
                "gchararray",
            ],
            "total_part_count": [
                21,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Part Count"),
                "gint",
            ],
            "revision_code": [
                22,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Revision Code"),
                "gchararray",
            ],
            "program_time": [
                23,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Program Time"),
                "gfloat",
            ],
            "program_time_sd": [
                24,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Program Time SE"),
                "gfloat",
            ],
            "program_cost": [
                25,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Total Cost"),
                "gfloat",
            ],
            "program_cost_sd": [
                26,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Total Cost SE"),
                "gfloat",
            ],
        }

        # Initialize public list class attributes.

        # Initialize public scalar class attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.tvwTreeView.set_tooltip_text(_("Displays the list of revisions."))

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._on_module_switch, "mvwSwitchedPage")

    def _on_module_switch(self, module: str = "") -> None:
        """Respond to changes in selected Module View module (tab).

        :param module: the name of the module that was just selected.
        :return: None
        """
        _model, _row = self.tvwTreeView.selection.get_selected()

        if module == self._tag and _row is not None:
            _code = _model.get_value(_row, self.tvwTreeView.position["revision_code"])
            _name = _model.get_value(_row, self.tvwTreeView.position["name"])
            _title = _(f"Analyzing Revision {_code}: {_name}")

            pub.sendMessage("request_set_title", title=_title)

    def _on_row_change(self, selection: Gtk.TreeSelection) -> None:
        """Handle events for the Revision package Module View RAMSTKTreeView().

        This method is called whenever a Revision Module View RAMSTKTreeView()
        row is activated/changed.

        :param selection: the Revision class Gtk.TreeSelection().
        :return: None
        """
        _attributes = super().on_row_change(selection)

        if _attributes:
            self._record_id = _attributes["revision_id"]

            _title = _("Analyzing Revision {0:s}: {1:s}").format(
                str(_attributes["revision_code"]), str(_attributes["name"])
            )

            pub.sendMessage("selected_revision", attributes=_attributes)
            pub.sendMessage("request_set_title", title=_title)


class RevisionGeneralDataPanel(RAMSTKFixedPanel):
    """The panel to display general data about the selected Revision."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field = "revision_id"
    _select_msg = "selected_revision"
    _tag = "revision"
    _title = _("General Revision Information")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Revision General Data panel."""
        super().__init__()

        # Initialize widgets.
        self.txtCode: RAMSTKEntry = RAMSTKEntry()
        self.txtName: RAMSTKEntry = RAMSTKEntry()
        self.txtRemarks: RAMSTKTextView = RAMSTKTextView(Gtk.TextBuffer())

        # Initialize private dict instance attributes.

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map = {
            "revision_code": [
                22,
                self.txtCode,
                "changed",
                super().on_changed_entry,
                "wvw_editing_revision",
                "",
                {
                    "width": 125,
                    "tooltip": _("A unique code for the selected revision."),
                },
                _("Revision Code:"),
                "gchararray",
            ],
            "name": [
                17,
                self.txtName,
                "changed",
                super().on_changed_entry,
                "wvw_editing_revision",
                "",
                {
                    "width": 800,
                    "tooltip": _("The name of the selected revision."),
                },
                _("Revision Name:"),
                "gchararray",
            ],
            "remarks": [
                20,
                self.txtRemarks,
                "changed",
                super().on_changed_textview,
                "wvw_editing_revision",
                "",
                {
                    "height": 100,
                    "width": 800,
                    "tooltip": _(
                        "Enter any remarks associated with the selected revision."
                    ),
                },
                _("Remarks:"),
                "gchararray",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel

class ResistorDesignElectricInputPanel(RAMSTKFixedPanel):
    _dic_quality: Dict[int, List[List[str]]]
    _dic_specifications: Dict[int, List[List[str]]]
    _dic_types: Dict[int, List[List[str]]]
    _dic_styles: Dict[int, Dict[int, List[List[str]]]]
    _dic_construction: Dict[int, List[List[str]]]
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    cmbConstruction: RAMSTKComboBox
    cmbQuality: RAMSTKComboBox
    cmbSpecification: RAMSTKComboBox
    cmbStyle: RAMSTKComboBox
    cmbType: RAMSTKComboBox
    txtNElements: RAMSTKEntry
    txtResistance: RAMSTKEntry
    _hazard_rate_method_id: int
    _quality_id: int
    dic_attribute_widget_map: Dict[str, List[Any]]
    category_id: int
    subcategory_id: int
    def __init__(self) -> None: ...
    def do_load_comboboxes(self, subcategory_id: int) -> None: ...
    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None: ...
    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_construction_combo(self) -> None: ...
    def __do_load_quality_combo(self) -> None: ...
    def __do_load_specification_combo(self) -> None: ...
    def __do_load_style_combo(self) -> None: ...
    def __do_load_type_combo(self) -> None: ...
    def __do_set_construction_combo_sensitive(
        self, attributes: Dict[str, Any]
    ) -> None: ...
    def __do_set_elements_entry_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def __do_set_specification_combo_sensitive(
        self, attributes: Dict[str, Any]
    ) -> None: ...
    def __do_set_style_combo_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def __do_set_type_combo_sensitive(self, attributes: Dict[str, Any]) -> None: ...



# Standard Library Imports
from typing import Any, Optional

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKDialog as RAMSTKDialog
from ramstk.views.gtk3.widgets import RAMSTKLabel as RAMSTKLabel

class AddStressTestMethod(RAMSTKDialog):
    rdoOpStress: Any = ...
    rdoTestMethod: Any = ...
    def __init__(self, parent: Optional[Any] = ...) -> None: ...
    def _cancel(self, __button: Gtk.Button) -> None: ...
    def __make_ui(self) -> None: ...
